@@ -31,7 +31,7 @@ use utf8;
 
 use Cwd qw(abs_path);
 use Getopt::Long();
-use List::MoreUtils qw(any);
+use List::MoreUtils qw(any none);
 use POSIX qw(:sys_wait_h);
 use Time::HiRes qw(gettimeofday tv_interval);
 
@@ -82,9 +82,8 @@ my @ENV_VARS = (
 ### "Normal" application variables
 my %conf_opt;                   #names of options set in the cfg file
 my %opt = (                     #hash of some flags from cmd or cfg
-    # Init some cmd-line value defaults
+     # Init some cmd-line value defaults
     'debug'             => 0,
-    'tag-display-limit' => 'auto',
 );
 
 my ($experimental_output_opts, $collmap, %overrides, $unpacker, @scripts);
@@ -92,7 +91,7 @@ my ($experimental_output_opts, $collmap, %overrides, $unpacker, @scripts);
 my ($STATUS_FD, @CLOSE_AT_END, $PROFILE, $TAGS);
 my @certainties = qw(wild-guess possible certain);
 my (@display_level, %display_source, %suppress_tags);
-my ($action, $checks, $check_tags, $dont_check);
+my ($action, $checks, $check_tags, $dont_check, $received_signal);
 my (@unpack_info, $LAB, %unpack_options, @auto_remove);
 my $user_dirs = $ENV{'LINTIAN_ENABLE_USER_DIRS'} // 1;
 my $exit_code = 0;
@@ -193,7 +192,9 @@ EOT-EOT-EOT
         # Not a special option per se, but most people will probably
         # not need it
         print <<"EOT-EOT-EOT";
+    --tag-display-limit X     Specify "tag per package" display limit
     --no-tag-display-limit    Disable "tag per package" display limit
+                              (equivalent to --tag-display-limit=0)
 EOT-EOT-EOT
     }
 
@@ -549,6 +550,7 @@ my %opthash = (
     'fail-on-warnings' => \$opt{'fail-on-warnings'},
     'keep-lab' => \$opt{'keep-lab'},
     'no-tag-display-limit' => sub { $opt{'tag-display-limit'} = 0; },
+    'tag-display-limit=i' => \$opt{'tag-display-limit'},
 
     # ------------------ configuration options
     'cfg=s' => \$opt{'LINTIAN_CFG'},
@@ -672,7 +674,7 @@ sub main {
     if (
         not(   ($action eq 'unpack')
             or ($action eq 'check'))
-      ) {
+    ) {
         fatal_error("invalid action $action specified");
     }
 
@@ -742,6 +744,11 @@ sub main {
     foreach my $gname (sort $pool->get_group_names) {
         my $success = 1;
         my $group = $pool->get_group($gname);
+
+        # Do not start a new group if we have a signal pending.
+        retrigger_signal() if $received_signal;
+
+        v_msg("Starting on group $gname");
         my $total_raw_res = timed_task {
             my @group_lpkg;
             my $raw_res = timed_task {
@@ -802,6 +809,7 @@ sub main {
         } else {
             print {$STATUS_FD} "error ${gname}${total_tres}\n";
         }
+        v_msg("Finished processing group $gname");
     }
 
     # Write the lab state to the disk, so it remembers the new packages
@@ -954,6 +962,8 @@ sub unpack_group {
     return
       unless $unpacker->prepare_tasks($errhandler, $group->get_processables);
 
+    retrigger_signal() if $received_signal;
+
     v_msg("Unpacking packages in group $gname");
 
     my (%timers, %hooks);
@@ -1040,6 +1050,7 @@ sub process_group {
             eval {$cs->run_check($lpkg, $group);};
             my $err = $@;
             my $raw_res = $finish_timer->($timer);
+            retrigger_signal() if $received_signal;
             if ($err) {
                 print STDERR $err;
                 print STDERR "internal error: cannot run $check check",
@@ -1234,6 +1245,7 @@ sub parse_config_file {
         'override'             => \&cfg_override,
         'show-overrides'       => \$opt{'show-overrides'},
         'suppress-tags'        => \&record_suppress_tags,
+        'tag-display-limit'    => \$opt{'tag-display-limit'},
         'verbose'              => \&cfg_verbosity,
     );
 
@@ -1289,7 +1301,9 @@ sub parse_config_file {
                 # Translate boolean strings to "0" or "1"; ignore
                 # errors as not all values are (intended to be)
                 # booleans.
-                eval { $val = parse_boolean($val); };
+                if (none { $var eq $_ } qw(jobs tag-display-limit)) {
+                    eval { $val = parse_boolean($val); };
+                }
                 if (ref $ref eq 'SCALAR'){
                     # Check it was already set
                     next if defined $$ref;
@@ -1415,8 +1429,8 @@ sub configure_output {
                 'The color value must be one of',
                 'never", "always", "auto" or "html"'));
     }
-    if ($opt{'tag-display-limit'} eq 'auto') {
-        if (-t STDOUT) {
+    if (not defined $opt{'tag-display-limit'}) {
+        if (-t STDOUT and not $opt{'verbose'}) {
             $opt{'tag-display-limit'}
               = Lintian::Output::DEFAULT_INTERACTIVE_TAG_LIMIT();
         } else {
@@ -1496,17 +1510,15 @@ sub setup_work_pool {
     for my $arg (@ARGV) {
         # file?
         if (-f $arg) {
-            if ($arg =~ m/\.(?:u?deb|dsc|changes)$/o){
+            if ($arg =~ m/\.(?:u?deb|dsc|changes|buildinfo)$/o){
                 eval {$pool->add_file($arg);};
                 if ($@) {
                     print STDERR "Skipping $arg: $@";
                     $exit_code = 2;
                 }
             } else {
-                fatal_error(
-                    join(q{ },
-                        "bad package file name $arg",
-                        '(neither .deb, .udeb, .changes or .dsc file)'));
+                fatal_error("bad package file name $arg (neither .deb, "
+                      . '.udeb, .changes .dsc or .buildinfo file)');
             }
         } else {
             # parameter is a package name--so look it up
@@ -1779,9 +1791,23 @@ sub END {
     }
 }
 
+sub _die_in_signal_handler {
+    die("N: Interrupted.\n");
+}
+
+sub retrigger_signal {
+    # Re-kill ourselves with the same signal to ensure that the exit
+    # code reflects that we died by a signal.
+    local $SIG{$received_signal} = \&_die_in_signal_handler;
+    debug_msg(2, "Retriggering signal SIG${received_signal}");
+    return kill($received_signal, $$);
+}
+
 sub interrupted {
-    $SIG{$_[0]} = 'DEFAULT';
-    die "N: Interrupted.\n";
+    $received_signal = $_[0];
+    $SIG{$received_signal} = 'DEFAULT';
+    print {$STATUS_FD} "ack-signal SIG${received_signal}\n";
+    return _die_in_signal_handler();
 }
 
 # }}}
